{

    Tool to stream MIDI data over TCP/IP or UDP/IP

    Copyright (C) 2003 .. 2008 Blue Hell / Jan Punter :: J.Punter@iaf.nl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; Version 2 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Modification history :

    2003-02-22 ported to D7 and published

}

Unit Data;

Interface

Uses

  Windows, SysUtils, Classes, SyncObjs, ContNrs, ScktComp, Math, mmSystem,
  Globals;

Type

{$WARN UNSAFE_TYPE OFF}
{$WARN UNSAFE_CODE OFF}
{$WARN UNSAFE_CAST OFF}

  EData                 = Class( Exception);
  EDataIndexOutOfBounds = Class( EData    );
  EDecoderState         = Class( EData    );
  EScheduler            = Class( EData    );

  TDataType = (
    dtNone,
    dtTime,
    dtMidi,
    dtAudio,
    dtChat
  );

  TData = Class;

  TOnDataComplete = Procedure( aSender: TData  ) Of Object;
  TOnOverflow     = Procedure( aSender: TObject) Of Object;
  TOnUnderflow    = Procedure( aSender: TObject) Of Object;

  TData = Class( TObject)
  Private
      FSize         : Word;
      FCount        : Word;
      FData         : Pointer;
      FDataType     : TDataType;
      FRelativeTime : Word;
  Private
    Procedure   CheckIndex( anIndex: Word);
    Procedure   SetSize( aValue: Word);
    Function    GetData( anIndex: Word): Byte;
    Procedure   SetData( anIndex: Word; aValue: Byte);
  Public
    Constructor Create( aSize: Word);                                  Overload;
    Constructor Create( aSize: Word; Const aData);                     Overload;
    Destructor  Destroy;                                               Override;
    Procedure   Dump( Const aFileName: String);
    Procedure   DumpAppend( Const aFileName: String);
    Function    AsDebugString: String;
    Procedure   Clear;                                                  Virtual;
    Procedure   AddByte( aByte: Byte);
    Procedure   AddByteEscaped( aByte: Byte);
    Procedure   AddData( aSize: Word; Const aData);
    Procedure   CopyFrom( aData: TData);
    Procedure   Encode;
    Procedure   SetCount( aValue: Integer);
  Public
    Property    Data[ anIndex: Word]: Byte      Read GetData       Write SetData;
    Property    Size                : Word      Read FSize         Write SetSize;
    Property    DataType            : TDataType Read FDataType     Write FDataType;
    Property    RelativeTime        : Word      Read FRelativeTime Write FRelativeTime;
    Property    RawData             : Pointer   Read FData;
    Property    Count               : Word      Read FCount;
  End;

  TDecoder = Class( TData)
  Private
      FOnDataComplete: TOnDataComplete;
      FState : (
        stStart,
        stStartDLE,
        stInType,
        stInTypeDLE,
        stLoTime,
        stLoTimeDLE,
        stHiTime,
        stHiTimeDLE,
        stInMSg,
        stDLE,
        stDone
      );
  Public
    Procedure   Clear;                                                 Override;
    Procedure   DecodeData( aSize: Word; Const aData);
  Public
    Property    OnDataComplete: TOnDataComplete Read FOnDataComplete Write FOnDataComplete;
  End;

  TDataArray = Array[ 0 .. MaxInt Div SizeOf( TData) - 1] Of TData;
  PDataArray = ^TDataArray;

  TTimedBuffer = Class;

  TTimedBufferThread = Class( TThread)
  Private
      FTimedBuffer : TTimedBuffer;
  Public
    Procedure   Execute;                                               Override;
  End;

  TTimedBuffer = Class( TObject)
  Private
      FTimerID           : Cardinal;
      FBufferTime        : Cardinal;
      FResolution        : Cardinal;
      FTimerEventHandle  : THandle;
      FLock              : TCriticalSection;
      FTimedBufferThread : TTimedBufferThread;
      FQueue             : PDataArray;
      FCount             : Integer;
      FFreeCount         : Integer;
      FFillCount         : Integer;
      FInPtr             : Integer;
      FOutPtr            : Integer;
      FOnDataComplete    : TOnDataComplete;
      FOnOverflow        : TOnOverflow;
      FOnUnderflow       : TOnUnderflow;
  Private
    Procedure   SetBufferTime( aValue: Cardinal);
    Procedure   SetResolution( aValue: Cardinal);
    procedure   StartTimer( aMs: Integer);
    Procedure   TimerFired( aSender: TObject);
    Procedure   ProcessData( aData: TData);
  Public
    Constructor Create( aCount: Integer);
    Destructor  Destroy;                                               Override;
    Procedure   Push( aData: TData);
    Function    Pop : TData;
    Function    Peek: TData;
  Public
    Property    Count         : Integer Read FCount;
    Property    FreeCount     : Integer Read FFreeCount;
    Property    FillCount     : Integer Read FFillCount;
  Public
    Property    BufferTime    : Cardinal        Read FBufferTime     Write SetBufferTime;
    Property    Resolution    : Cardinal        Read FResolution     Write SetResolution;
    Property    OnDataComplete: TOnDataComplete Read FOnDataComplete Write FOnDataComplete;
    Property    OnOverflow    : TOnOverflow     Read FOnOverflow     Write FOnOverflow;
    Property    OnUnderflow   : TOnUnderflow    Read FOnUnderflow    Write FOnUnderflow;
  End;

  Function DataTypeToStr( aDataType: TDataType): String;

Implementation

  Function DataTypeToStr( aDataType: TDataType): String;
  Begin
    Case aDataType Of
      dtNone  : Result := 'None';
      dtTime  : Result := 'Time';
      dtMidi  : Result := 'Midi';
      dtAudio : Result := 'Audio';
      dtChat  : Result := 'Chat';
      Else      Result := 'Unknown';
    End;
  End;



Type

  TBytes = Array[ 0 .. MaxInt Div SizeOf( Byte) - 1] Of Byte;
  PBytes = ^TBytes;


{ ========
  TData = Class( TObject)
  Private
      FSize         : Word;
      FCount        : Word;
      FData         : Pointer;
      FDataType     : TDataType;
      FRelativeTime : Word;
  Public
    Property    Data[ anIndex: Word]: Byte      Read GetData       Write SetData;
    Property    Size                : Word      Read FSize         Write SetSize;
    Property    DataType            : TDataType Read FDataType     Write FDataType;
    Property    RelativeTime        : Word      Read FRelativeTime Write FRelativeTime;
    Property    RawData             : Pointer   Read FData;
    Property    Count               : Word      Read FCount;
  Private
}

    Procedure   TData.CheckIndex( anIndex: Word);
    Begin
      If anIndex >= Size
      Then
        Raise
          EDataIndexOutOfBounds.
            CreateFmt(
              'TData : Index (%d) out of bounds [0,%d]',
              [
                anIndex,
                Size - 1
              ]
            );
    End;

    Procedure   TData.SetSize( aValue: Word);
    Var
      NewData : Pointer;
    Begin
      If aValue <> FSize
      Then Begin
        NewData := Nil;
        If aValue > 0
        Then Begin
          GetMem( NewData, aValue);
          FillChar( NewData^, aValue, 0);
          If FSize > 0
          Then Move( FData^, NewData^, Min( FSize, aValue));
        End;
        If FSize > 0
        Then FreeMem( FData, FSize);
        FData := NewData;
        FSize := aValue;
      End;
    End;

    Function    TData.GetData( anIndex: Word): Byte;
    Begin
      CheckIndex( anIndex);
      Result := PBytes( FData)^[ anIndex];
    End;

    Procedure   TData.SetData( anIndex: Word; aValue: Byte);
    Begin
      CheckIndex( anIndex);
      PBytes( FData)^[ anIndex] := aValue;
    End;

//  Public

    Constructor TData.Create( aSize: Word); // Overload;
    Begin
      Inherited Create;
      Size := aSize;
      Clear;
    End;

    Constructor TData.Create( aSize: Word; Const aData); // Overload;
    Begin
      Create( aSize);
      AddData( aSize, aData);
    End;

    Destructor  TData.Destroy; // Override;
    Begin
      Size := 0;
      Inherited;
    End;

    Procedure   TData.Dump( Const aFileName: String);
    Var
      aFile : TextFile;
    Begin
      AssignFile( aFile, aFileName);
      Rewrite( aFile);
      Try
        WriteLn( aFile, AsDebugString);
      Finally
        CloseFile( aFile);
      End;
    End;

    Procedure   TData.DumpAppend( Const aFileName: String);
    Var
      aFile : TextFile;
    Begin
      AssignFile( aFile, aFileName);
      If FileExists( aFileName)
      Then Append( aFile)
      Else Rewrite( aFile);
      Try
        WriteLn( aFile, AsDebugString);
      Finally
        CloseFile( aFile);
      End;
    End;

    Function    TData.AsDebugString: String;
    Var
      i : Integer;
    Begin
      Result :=
        Format(
          '%.5d %s : [',
          [
            RelativeTime,
            TrimStr( DataTypeToStr( DataType), 7)
          ]
        );
      For i := 0 To Count - 1 Do
      Begin
        Result := Result + Format( '%.2x,', [ Data[ i]]);
        If Length( Result) > 1024
        Then Break;
      End;
      If Length( Result) < 1024
      Then
        If Count > 0
        Then Result[ Length( Result)] := ']'
        Else Result := Result + ']';
    End;

    Procedure   TData.Clear;
    Begin
      FCount := 0;
    End;

    Procedure   TData.AddByte( aByte: Byte);
    Begin
      If FSize = 0
      Then Size := 1
      Else If FCount >= FSize
      Then Size := Size * 2;
      Move( aByte, PBytes( FData)^[ FCount], 1);
      Inc( FCount);
    End;

Const

  STX =  2;
  ETX =  3;
  DLE = 16;

    Procedure   TData.AddByteEscaped( aByte: Byte);
    Begin
      If aByte In [ DLE]
      Then AddByte( DLE);
      AddByte( aByte);
    End;

    Procedure   TData.AddData( aSize: Word; Const aData);
    Begin
      If aSize > 0
      Then Begin
        If FSize = 0
        Then Size := aSize
        Else
          While Count + aSize > Size Do
            Size := FSize * 2;
        Move( aData, PBytes( FData)^[ FCount], aSize);
        Inc( FCount, aSize);
      End;
    End;


    Procedure   TData.CopyFrom( aData: TData);
    Begin
      If Assigned( aData)
      Then Begin
        Clear;
        AddData( aData.Count, aData.FData^);
        DataType     := aData.DataType;
        RelativeTime := aData.RelativeTime;
      End;
    End;

    Procedure   TData.Encode;
    Var
      aData : TData;
      i     : Integer;
    Begin
      aData := TData.Create( 128); // This will be autosized if too small.
      With aData Do
      Begin
        DataType     := Self.DataType;
        RelativeTime := Self.RelativeTime;
        AddByte( DLE);
        AddByte( STX);
      End;
      aData.AddByteEscaped( Ord( DataType));
      aData.AddByteEscaped( Lo( RelativeTime));
      aData.AddByteEscaped( Hi( RelativeTime));
      For i := 0 To Count - 1 Do
        aData.AddByteEscaped( Data[ i]);
      With aData Do
      Begin
        AddByte( DLE);
        AddByte( ETX);
      End;
      CopyFrom( aData);
      aData.Free;
    End;

    Procedure   TData.SetCount( aValue: Integer);
    Begin
      CheckIndex( aValue - 1);
      FCount := aValue;
    End;


{ =========
  TDecoder = Class( TData)
  Private
      FOnDataComplete: TOnDataComplete;
      FState : (
        stStart,
        stStartDLE,
        stInType,
        stInTypeDLE,
        stLoTime,
        stLoTimeDLE,
        stHiTime,
        stHiTimeDLE
        stInMSg,
        stDLE,
        stDone
      );
  Public
    Property    OnDataComplete: TOnDataComplete Read FOnDataComplete Write FOnDataComplete;
}

//   Public

    Procedure   TDecoder.Clear; // Override;
    Begin
      Inherited;
      FState := stStart;
    End;

    Procedure   TDecoder.DecodeData( aSize: Word; Const aData); // Override;
    Var
      i     : Integer;
      aByte : Byte;
      Data  : TData;
    Begin
      For i := 0 To aSize - 1 Do
      Begin
        aByte := PBytes( @ aData)^[ i];
        Case FState Of
          stStart    :
            If aByte = DLE
            Then FState := stStartDLE;
          stStartDLE :
            If aByte = STX
            Then FState := stInType
            Else FState := stStart;
          stInType :
            If aByte = DLE
            Then FState := stInTypeDLE
            Else Begin
              DataType := TDataType( aByte);
              FState   := stLoTime;
            End;
          stInTypeDLE :
            If aByte = DLE
            Then Begin
              DataType := TDataType( DLE);
              FState   := stLoTime;
            End
            Else Clear;
          stLoTime :
            If aByte = DLE
            Then FState := stLoTimeDLE
            Else Begin
              RelativeTime := aByte;
              FState       := stHiTime;
            End;
          stLoTimeDLE :
            If aByte = DLE
            Then Begin
              RelativeTime := DLE;
              FState       := stHiTime;
            End
            Else Clear;
          stHiTime :
            If aByte = DLE
            Then FState := stHiTimeDLE
            Else Begin
              RelativeTime := RelativeTime + 256 * aByte;
              FState       := stInMsg;
            End;
          stHiTimeDLE :
            If aByte = DLE
            Then Begin
              RelativeTime := RelativeTime + 256 * DLE;
              FState       := stInMsg;
            End
            Else Clear;
          stInMSg :
            If aByte = DLE
            Then FState := stDLE
            Else AddByte( aByte);
          stDLE :
            Case aByte Of
              STX : FState := stInMsg; // an error, but lets try to resync ...
              ETX : Begin
                  FState := stDone;    // End of message, call callback
                  Data   := TData.Create( Count);
                  If Assigned( FOnDataComplete)
                  Then
                    Try
                      Data.CopyFrom( Self);
                      FOnDataComplete( Data);
                    Finally
                      Data.Free;
                      Clear;
                    End;
                End;
              DLE : Begin
                  AddByte( DLE);
                  FState := stInMsg;
                End;
            End;
          Else Raise EDecoderState.Create( 'TDecoder.AddData : Invalid state');
        End;
      End;
    End;

{ ========
  TTimedBufferThread = Class( TThread)
  Private
      FTimedBuffer : TTimedBuffer;
  Public
}

  Procedure TimerCallback( uTimerID, uMessage: UINT; dwUser, dw1, dw2: DWORD); stdcall;
  Var
    aTimedBuffer: TTimedBuffer;
  Begin
    aTimedBuffer := TTimedBuffer( dwUser);
    If
      ( uTimerID <> 0) And
      Assigned( aTimedBuffer) And
      ( aTimedBuffer.FTimerEventHandle <> 0)
    Then SetEvent( aTimedBuffer.FTimerEventHandle);
  End;


    Procedure   TTImedBufferThread.Execute; // Override;
    Begin
      While Not Terminated And Assigned( FTimedBuffer) Do
      Begin
        WaitForSingleObject( FTimedBuffer.FTimerEventHandle, INFINITE);
        Try
          If Not Terminated
          Then FTimedBuffer.TimerFired( Self);
        Except
          On Exception Do
            ;
        End;
      End;
    End;

{ ========
  TTimedBuffer = Class( TObject)
  Private
      FTimerID           : UINT;
      FResolution        : UINT;
      FTimerEventHandle  : THandle;
      FTimedBufferThread : TTimedBufferThread;
      FLock              : TCriticalSection;
      FQueue             : PDataArray;
      FCount             : Integer;
      FFreeCount         : Integer;
      FFillCount         : Integer;
      FInPtr             : Integer;
      FOutPtr            : Integer;
      FBufferTime        : Integer;
      FOnDataComplete    : TOnDataComplete;
      FOnOverflow        : TOnOverflow;
      FOnUnderflow       : TOnUnderflow;
  Public
    Property    Count         : Integer Read FCount;
    Property    FreeCount     : Integer Read FFreeCount;
    Property    FillCount     : Integer Read FFillCount;
  Public
    Property    BufferTime    : Integer         Read FBufferTime     Write FBufferTime;
    Property    Resolution    : Integer         Read FResolution     Write FResolution;
    Property    OnDataComplete: TOnDataComplete Read FOnDataComplete Write FOnDataComplete;
    Property    OnOverflow    : TOnOverflow     Read FOnOverflow     Write FOnOverflow;
    Property    OnUnderflow   : TOnUnderflow    Read FOnUnderflow    Write FOnUnderflow;
  Private
}

    Procedure   TTimedBuffer.SetBufferTime( aValue: Cardinal);
    Begin
      If aValue < 1
      Then FBufferTime := 1
      Else If aValue > 60000
      Then FBufferTime := 60000
      Else FBufferTime := aValue;
    End;

    Procedure   TTimedBuffer.SetResolution( aValue: Cardinal);
    Begin
      If aValue < 1
      Then FResolution := 1
      Else If aValue > 1000
      Then FResolution := 1000
      Else FResolution := aValue;
    End;

    procedure   TTimedBuffer.StartTimer( aMs: Integer);
    Var
      aTimerProc: TFNTimeCallBack;
    Begin
      aTimerProc := @ TimerCallback;
      FTimerID := timeSetEvent( aMs, FResolution, aTimerProc, DWORD( self), TIME_ONESHOT);
      If FTimerID = 0
      Then Raise EScheduler.Create( 'Out of timer resources');
    End;

    Procedure   TTimedBuffer.TimerFired( aSender: TObject);
    Var
      CurrData : TData;
      NextData : TData;
    Begin
      // The current event fired the timer, get it and process it.
      CurrData := Pop;
      While Assigned( CurrData) Do
      Begin
        // Invariant : CurrData is the oldest data that can be processed.
        NextData := Peek;
        // The next event determines when the timer has to fire again, if it has to
        If Assigned( NextData)
        Then Begin
          If NextData.RelativeTime > 10
          Then Begin
            StartTimer( NextData.RelativeTime - 10); // Minus the time spent here,
                                                //   which is an unknown amount ...
            NextData := Nil;                    // Assert loop quiting
          End
          Else NextData := Pop;                 // Stay in loop, keep loop invariant
        End;
        ProcessData( CurrData);
        CurrData.Free;                          // Current data is processed, abandon it.
        CurrData := NextData;                   // Propagate loop invariant.
      End;
    End;

    Procedure   TTimedBuffer.ProcessData( aData: TData);
    Begin
      If Assigned( FOnDataComplete) And Assigned( aData)
      Then FOnDataComplete( aData);
    End;

//   Public

    Constructor TTimedBuffer.Create( aCount: Integer);
    Begin
      Inherited Create;

      GetMem( FQueue, aCount * SizeOf( TData));
      FCount      := aCount;
      FInPtr      := 0;
      FOutPtr     := 0;
      FFreeCount  := FCount;
      FFillCount  := 0;
      FBufferTime := 1000; // ms
      FResolution := 20;   // ms

      FLock              := TCriticalSection.Create;
      FTimerEventHandle  := CreateEvent( Nil, False, False, Nil);
      FTimedBufferThread := TTimedBufferThread.Create( True);
      FTimedBufferThread.Priority := tpHigher;
      With FTimedBufferThread Do
      Begin
        FTimedBuffer    := Self;
        FreeOnTerminate := True;
        Resume;
      End;
   End;

    Destructor  TTimedBuffer.Destroy; // Override;
    Var
      aHandle : THandle;
    Begin
      FTimedBufferThread.Terminate; // Which will free the thread
      aHandle := FTimerEventHandle;
      FTimerEventHandle := 0;
      CloseHandle( aHandle);
      While FFillCount > 0 Do
        Pop.Free;
      FLock.Free;
      FreeMem( FQueue, FCount * SizeOf( TData));
      Inherited;
    End;

    Procedure   TTimedBuffer.Push( aData: TData);
    Begin
      FLock.Enter;
      Try
        If FFreeCount > 0
        Then Begin
          Dec( FFreeCount);
          FQueue[ FInPtr] := aData;
          Inc( FInPtr);
          If FInPtr >= FCount
          Then FInPtr := 0;
          Inc( FFillCount);
          If FFillCount = 1
          Then StartTimer( FBufferTime);
        End
        Else
          If Assigned( FOnOverflow)
          Then FOnOverflow( Self);
      Finally
        FLock.Leave;
      End;
    End;

    Function    TTimedBuffer.Pop: TData;
    Begin
      FLock.Enter;
      Try
        If FFillCount > 0
        Then Begin
          Dec( FFillCount);
          Result := FQueue[ FOutPtr];
          Inc( FOutPtr);
          If FOutPtr >= FCount
          Then FOutPtr := 0;
          Inc( FFreeCount);
        End
        Else Result := Nil;
      Finally
        FLock.Leave;
      End;
    End;

    Function    TTimedBuffer.Peek: TData;
    Begin
      FLock.Enter;
      Try
        If FFillCount > 0
        Then Result := FQueue[ FOutPtr]
        Else Result := Nil;
      Finally
        FLock.Leave;
      End;
    End;



End.
