unit WinSockErrors;

interface


  function SocketErrorInfo( anErrNo: integer): string;


implementation

uses

  SysUtils;

type

  TSocketErrorInfo = record
    ErrNo     : Integer;
    ShortText : string;
  end;


const

  SocketErrorInfos : array[ 0 .. 94] of TSocketErrorInfo =
  (
    ( ErrNo: { WSA_INVALID_HANDLE         }     6 ; ShortText: 'Specified event object handle is invalid.'         ),
    ( ErrNo: { WSA_NOT_ENOUGH_MEMORY      }     8 ; ShortText: 'Insufficient memory available.'                    ),
    ( ErrNo: { WSA_INVALID_PARAMETER      }    87 ; ShortText: 'One or more parameters are invalid.'               ),
    ( ErrNo: { WSA_OPERATION_ABORTED      }   995 ; ShortText: 'Overlapped operation aborted.'                     ),
    ( ErrNo: { WSA_IO_INCOMPLETE          }   996 ; ShortText: 'Overlapped I/O event object not in signaled state.'),
    ( ErrNo: { WSA_IO_PENDING             }   997 ; ShortText: 'Overlapped operations will complete later.'        ),
    ( ErrNo: { WSAEINTR                   } 10004 ; ShortText: 'Interrupted function call.'                        ),
    ( ErrNo: { WSAEBADF                   } 10009 ; ShortText: 'File handle is not valid.'                         ),
    ( ErrNo: { WSAEACCES                  } 10013 ; ShortText: 'Permission denied.'                                ),
    ( ErrNo: { WSAEFAULT                  } 10014 ; ShortText: 'Bad address.'                                      ),

    ( ErrNo: { WSAEINVAL                  } 10022 ; ShortText: 'Invalid argument.'                                 ),
    ( ErrNo: { WSAEMFILE                  } 10024 ; ShortText: 'Too many open files.'                              ),
    ( ErrNo: { WSAEWOULDBLOCK             } 10035 ; ShortText: 'Resource temporarily unavailable.'                 ),
    ( ErrNo: { WSAEINPROGRESS             } 10036 ; ShortText: 'Operation now in progress.'                        ),
    ( ErrNo: { WSAEALREADY                } 10037 ; ShortText: 'Operation already in progress.'                    ),
    ( ErrNo: { WSAENOTSOCK                } 10038 ; ShortText: 'Socket operation on nonsocket.'                    ),
    ( ErrNo: { WSAEDESTADDRREQ            } 10039 ; ShortText: 'Destination address required.'                     ),
    ( ErrNo: { WSAEMSGSIZE                } 10040 ; ShortText: 'Message too long.'                                 ),
    ( ErrNo: { WSAEPROTOTYPE              } 10041 ; ShortText: 'Protocol wrong type for socket.'                   ),
    ( ErrNo: { WSAENOPROTOOPT             } 10042 ; ShortText: 'Bad protocol option.'                              ),

    ( ErrNo: { WSAEPROTONOSUPPORT         } 10043 ; ShortText: 'Protocol not supported.'                           ),
    ( ErrNo: { WSAESOCKTNOSUPPORT         } 10044 ; ShortText: 'Socket type not supported.'                        ),
    ( ErrNo: { WSAEOPNOTSUPP              } 10045 ; ShortText: 'Operation not supported.'                          ),
    ( ErrNo: { WSAEPFNOSUPPORT            } 10046 ; ShortText: 'Protocol family not supported.'                    ),
    ( ErrNo: { WSAEAFNOSUPPORT            } 10047 ; ShortText: 'Address family not supported by protocol family.'  ),
    ( ErrNo: { WSAEADDRINUSE              } 10048 ; ShortText: 'Address already in use.'                           ),
    ( ErrNo: { WSAEADDRNOTAVAIL           } 10049 ; ShortText: 'Cannot assign requested address.'                  ),
    ( ErrNo: { WSAENETDOWN                } 10050 ; ShortText: 'Network is down.'                                  ),
    ( ErrNo: { WSAENETUNREACH             } 10051 ; ShortText: 'Network is unreachable.'                           ),
    ( ErrNo: { WSAENETRESET               } 10052 ; ShortText: 'Network dropped connection on reset.'              ),

    ( ErrNo: { WSAECONNABORTED            } 10053 ; ShortText: 'Software caused connection abort.'                 ),
    ( ErrNo: { WSAECONNRESET              } 10054 ; ShortText: 'Connection reset by peer.'                         ),
    ( ErrNo: { WSAENOBUFS                 } 10055 ; ShortText: 'No buffer space available.'                        ),
    ( ErrNo: { WSAEISCONN                 } 10056 ; ShortText: 'Socket is already connected.'                      ),
    ( ErrNo: { WSAENOTCONN                } 10057 ; ShortText: 'Socket is not connected.'                          ),
    ( ErrNo: { WSAESHUTDOWN               } 10058 ; ShortText: 'Cannot send after socket shutdown.'                ),
    ( ErrNo: { WSAETOOMANYREFS            } 10059 ; ShortText: 'Too many references.'                              ),
    ( ErrNo: { WSAETIMEDOUT               } 10060 ; ShortText: 'Connection timed out.'                             ),
    ( ErrNo: { WSAECONNREFUSED            } 10061 ; ShortText: 'Connection refused.'                               ),
    ( ErrNo: { WSAELOOP                   } 10062 ; ShortText: 'Cannot translate name.'                            ),

    ( ErrNo: { WSAENAMETOOLONG            } 10063 ; ShortText: 'Name too long.'                                    ),
    ( ErrNo: { WSAEHOSTDOWN               } 10064 ; ShortText: 'Host is down.'                                     ),
    ( ErrNo: { WSAEHOSTUNREACH            } 10065 ; ShortText: 'No route to host.'                                 ),
    ( ErrNo: { WSAENOTEMPTY               } 10066 ; ShortText: 'Directory not empty.'                              ),
    ( ErrNo: { WSAEPROCLIM                } 10067 ; ShortText: 'Too many processes.'                               ),
    ( ErrNo: { WSAEUSERS                  } 10068 ; ShortText: 'User quota exceeded.'                              ),
    ( ErrNo: { WSAEDQUOT                  } 10069 ; ShortText: 'Disk quota exceeded.'                              ),
    ( ErrNo: { WSAESTALE                  } 10070 ; ShortText: 'Stale file handle reference.'                      ),
    ( ErrNo: { WSAEREMOTE                 } 10071 ; ShortText: 'Item is remote.'                                   ),
    ( ErrNo: { WSASYSNOTREADY             } 10091 ; ShortText: 'Network subsystem is unavailable.'                 ),

    ( ErrNo: { WSAVERNOTSUPPORTED         } 10092 ; ShortText: 'Winsock.dll version out of range.'                 ),
    ( ErrNo: { WSANOTINITIALISED          } 10093 ; ShortText: 'Successful WSAStartup not yet performed.'          ),
    ( ErrNo: { WSAEDISCON                 } 10101 ; ShortText: 'Graceful shutdown in progress.'                    ),
    ( ErrNo: { WSAENOMORE                 } 10102 ; ShortText: 'No more results.'                                  ),
    ( ErrNo: { WSAECANCELLED              } 10103 ; ShortText: 'Call has been canceled.'                           ),
    ( ErrNo: { WSAEINVALIDPROCTABLE       } 10104 ; ShortText: 'Procedure call table is invalid.'                  ),
    ( ErrNo: { WSAEINVALIDPROVIDER        } 10105 ; ShortText: 'Service provider is invalid.'                      ),
    ( ErrNo: { WSAEPROVIDERFAILEDINIT     } 10106 ; ShortText: 'Service provider failed to initialize.'            ),
    ( ErrNo: { WSASYSCALLFAILURE          } 10107 ; ShortText: 'System call failure.'                              ),
    ( ErrNo: { WSASERVICE_NOT_FOUND       } 10108 ; ShortText: 'Service not found.'                                ),

    ( ErrNo: { WSATYPE_NOT_FOUND          } 10109 ; ShortText: 'Class type not found.'                             ),
    ( ErrNo: { WSA_E_NO_MORE              } 10110 ; ShortText: 'No more results.'                                  ),
    ( ErrNo: { WSA_E_CANCELLED            } 10111 ; ShortText: 'Call was canceled.'                                ),
    ( ErrNo: { WSAEREFUSED                } 10112 ; ShortText: 'Database query was refused.'                       ),
    ( ErrNo: { WSAHOST_NOT_FOUND          } 11001 ; ShortText: 'Host not found.'                                   ),
    ( ErrNo: { WSATRY_AGAIN               } 11002 ; ShortText: 'Nonauthoritative host not found.'                  ),
    ( ErrNo: { WSANO_RECOVERY             } 11003 ; ShortText: 'This is a nonrecoverable error.'                   ),
    ( ErrNo: { WSANO_DATA                 } 11004 ; ShortText: 'Valid name, no data record of requested type.'     ),
    ( ErrNo: { WSA_QOS_RECEIVERS          } 11005 ; ShortText: 'QOS receivers.'                                    ),
    ( ErrNo: { WSA_QOS_SENDERS            } 11006 ; ShortText: 'QOS senders.'                                      ),

    ( ErrNo: { WSA_QOS_NO_SENDERS         } 11007 ; ShortText: 'No QOS senders.'                                   ),
    ( ErrNo: { WSA_QOS_NO_RECEIVERS       } 11008 ; ShortText: 'QOS no receivers.'                                 ),
    ( ErrNo: { WSA_QOS_REQUEST_CONFIRMED  } 11009 ; ShortText: 'QOS request confirmed.'                            ),
    ( ErrNo: { WSA_QOS_ADMISSION_FAILURE  } 11010 ; ShortText: 'QOS admission error.'                              ),
    ( ErrNo: { WSA_QOS_POLICY_FAILURE     } 11011 ; ShortText: 'QOS policy failure.'                               ),
    ( ErrNo: { WSA_QOS_BAD_STYLE          } 11012 ; ShortText: 'QOS bad style.'                                    ),
    ( ErrNo: { WSA_QOS_BAD_OBJECT         } 11013 ; ShortText: 'QOS bad object.'                                   ),
    ( ErrNo: { WSA_QOS_TRAFFIC_CTRL_ERROR } 11014 ; ShortText: 'QOS traffic control error.'                        ),
    ( ErrNo: { WSA_QOS_GENERIC_ERROR      } 11015 ; ShortText: 'QOS generic error.'                                ),
    ( ErrNo: { WSA_QOS_ESERVICETYPE       } 11016 ; ShortText: 'QOS service type error.'                           ),

    ( ErrNo: { WSA_QOS_EFLOWSPEC          } 11017 ; ShortText: 'QOS flowspec error.'                               ),
    ( ErrNo: { WSA_QOS_EPROVSPECBUF       } 11018 ; ShortText: 'Invalid QOS provider buffer.'                      ),
    ( ErrNo: { WSA_QOS_EFILTERSTYLE       } 11019 ; ShortText: 'Invalid QOS filter style.'                         ),
    ( ErrNo: { WSA_QOS_EFILTERTYPE        } 11020 ; ShortText: 'Invalid QOS filter type.'                          ),
    ( ErrNo: { WSA_QOS_EFILTERCOUNT       } 11021 ; ShortText: 'Incorrect QOS filter count.'                       ),
    ( ErrNo: { WSA_QOS_EOBJLENGTH         } 11022 ; ShortText: 'Invalid QOS object length.'                        ),
    ( ErrNo: { WSA_QOS_EFLOWCOUNT         } 11023 ; ShortText: 'Incorrect QOS flow count.'                         ),
    ( ErrNo: { WSA_QOS_EUNKOWNPSOBJ       } 11024 ; ShortText: 'Unrecognized QOS object.'                          ),
    ( ErrNo: { WSA_QOS_EPOLICYOBJ         } 11025 ; ShortText: 'Invalid QOS policy object.'                        ),
    ( ErrNo: { WSA_QOS_EFLOWDESC          } 11026 ; ShortText: 'Invalid QOS flow descriptor.'                      ),

    ( ErrNo: { WSA_QOS_EPSFLOWSPEC        } 11027 ; ShortText: 'Invalid QOS provider-specific flowspec.'           ),
    ( ErrNo: { WSA_QOS_EPSFILTERSPEC      } 11028 ; ShortText: 'Invalid QOS provider-specific filterspec.'         ),
    ( ErrNo: { WSA_QOS_ESDMODEOBJ         } 11029 ; ShortText: 'Invalid QOS shape discard mode object.'            ),
    ( ErrNo: { WSA_QOS_ESHAPERATEOBJ      } 11030 ; ShortText: 'Invalid QOS shaping rate object.'                  ),
    ( ErrNo: { WSA_QOS_RESERVED_PETYPE    } 11031 ; ShortText: 'Reserved policy QOS element type.'                 )
  );


  function SocketErrorInfo( anErrNo: integer): string;
  var
    i : Integer;
  begin
    Result := Format( '%d', [ anErrNo]);
    for i := Low( SocketErrorInfos) to High( SocketErrorInfos)
    do begin
      with SocketErrorInfos[ i]
      do begin
        if ErrNo = anErrNo
        then begin
          Result := ShortText;
          Break;
        end;
      end;
    end;
  end;


end.
